@@ -3,7 +3,7 @@
 #include "types.h"
 #define PROC_CPT	"/proc/cpt"
 #define PROC_RST	"/proc/rst"
-#define DEF_DUMPDIR	"/vz/dump"
+#define DEF_DUMPDIR	"/var/tmp"
 
 #define CMD_CHKPNT		1
 #define CMD_SUSPEND		2
