@@ -19,6 +19,9 @@
 #define	_UTIL_H_
 
 #include "types.h"
+#define BACKUP		0
+#define DESTR		1
+
 #define PROCMEM		"/proc/meminfo"
 #define PROCTHR		"/proc/sys/kernel/threads-max"
 
@@ -41,6 +44,7 @@ int get_pagesize();
 int get_mem(unsigned long long *mem);
 int get_thrmax(int *thrmax);
 int get_swap(unsigned long long *swap);
+int get_num_cpu();
 int get_lowmem(unsigned long long *mem);
 unsigned long max_ul(unsigned long val1, unsigned long val2);
 void str_tolower(const char *from, char *to);
@@ -48,5 +52,5 @@ char *get_file_name(char *str);
 const char *get_vps_state_str(int vps_state);
 int set_not_blk(int fd);
 void close_fds(int close_std, ...);
-
+int move_config(int veid, int action);
 #endif
