@@ -38,7 +38,7 @@
 #define PARAM_NUMOTHERSOCK	10
 #define PARAM_NUMIPTENT		'e'
 #define PARAM_NUMFILE		'n'
-#define PARAM_DCACHE		'x'
+#define PARAM_DCACHESIZE	'x'
 
 #define PARAM_PKGVER		11
 #define PARAM_PKGSET		12
@@ -80,6 +80,7 @@
 #define PARAM_TEMPLATES		93
 #define PARAM_DEVNODES		94 
 #define	PARAM_SKIPQUOTACHECK	95
+#define PARAM_MEMINFO		97
 
 #define PARAM_FAST		310
 #define PARAM_APPLYCONFIG	311
@@ -109,6 +110,9 @@
 #define PARAM_UNDUMP		337
 #define PARAM_DUMPDIR		338
 #define PARAM_SKIPARPDETECT	339
+#define PARAM_RESET_UB		340
+#define PARAM_VETH_ADD		341
+#define PARAM_VETH_DEL		342
 
 #define PARAM_LINE		"e:p:f:t:i:l:k:a:b:n:x:h"
 #endif
