@@ -141,6 +141,8 @@
 #define	VZ_NO_DISTR_ACTION_SCRIPT	108
 #define VZ_APPLY_CONFIG_ERROR		109
 #define VZ_CUSTOM_REINSTALL_ERROR	128
+#define VZ_SET_MEMINFO_ERROR		129
+#define VZ_VETH_ERROR			130
 
 #endif /* _VZ_ERROR_H_ */
 
