@@ -39,7 +39,7 @@
 #include "vzctl.h"
 #include "res.h"
 #include "iptables.h"
-
+#include "meminfo.h"
 
 static int _page_size;
 
@@ -72,7 +72,7 @@ static vps_config config[] = {
 {"DGRAMRCVBUF",	NULL, PARAM_DGRAMRCVBUF},
 {"NUMOTHERSOCK",NULL, PARAM_NUMOTHERSOCK},
 {"NUMFILE",	NULL, PARAM_NUMFILE},
-{"DCACHESIZE",	NULL, PARAM_DCACHE},
+{"DCACHESIZE",	NULL, PARAM_DCACHESIZE},
 {"DCACHE",	"DCACHESIZE", -1},
 {"NUMIPTENT",	NULL, PARAM_NUMIPTENT},
 {"IPTENTRIES",	"NUMIPTENT", -1},
@@ -114,6 +114,8 @@ static vps_config config[] = {
 {"DISKINODES",	NULL, PARAM_DISKINODES},
 {"QUOTATIME",	NULL, PARAM_QUOTATIME},
 {"QUOTAUGIDLIMIT",NULL, PARAM_QUOTAUGIDLIMIT},
+{"MEMINFO",	NULL, PARAM_MEMINFO},
+{"VETH",	NULL, PARAM_VETH_ADD},
 
 {NULL		,NULL, -1}
 };
@@ -122,6 +124,7 @@ static struct option set_opt[] = {
 {"save",	no_argument, NULL, PARAM_SAVE},
 {"applyconfig",	required_argument, NULL, PARAM_APPLYCONFIG},
 {"config",	required_argument, NULL, PARAM_CONFIG},
+{"reset_ub",	no_argument, NULL, PARAM_RESET_UB},
 {"iptables",	required_argument, NULL, PARAM_IPTABLES},
 /*	UB	*/
 {"kmemsize",	required_argument, NULL, PARAM_KMEMSIZE},
@@ -142,7 +145,7 @@ static struct option set_opt[] = {
 {"dgramrcvbuf",	required_argument, NULL, PARAM_DGRAMRCVBUF},
 {"numothersockk",required_argument, NULL, PARAM_NUMOTHERSOCK},
 {"numfile",	required_argument, NULL, PARAM_NUMFILE},
-{"dcachesize",	required_argument, NULL, PARAM_DCACHE},
+{"dcachesize",	required_argument, NULL, PARAM_DCACHESIZE},
 {"numiptent",	required_argument, NULL, PARAM_NUMIPTENT},
 {"avnumproc",	required_argument, NULL, PARAM_AVNUMPROC},
 /*	Capability */
@@ -182,6 +185,10 @@ static struct option set_opt[] = {
 {"diskinodes",	required_argument, NULL, PARAM_DISKINODES},
 {"quotatime",	required_argument, NULL, PARAM_QUOTATIME},
 {"quotaugidlimit", required_argument, NULL, PARAM_QUOTAUGIDLIMIT},
+{"meminfo",	required_argument, NULL, PARAM_MEMINFO},
+/*	veth	*/
+{"veth_add",	required_argument, NULL, PARAM_VETH_ADD},
+{"veth_del",	required_argument, NULL, PARAM_VETH_DEL},
 
 {NULL, 0, NULL, 0}
 };
@@ -502,6 +509,55 @@ int parse_twoul(const char *str, unsigned long *val)
 	}
 	return ret;
 }
+/******************** totalmem *************************/
+int parse_meminfo(meminfo_param *param, char *val)
+{
+	int mode;
+	char mode_nm[32];
+	unsigned long meminfo_val;
+	int ret;
+
+	if (*val == 0)
+		return 0;
+	meminfo_val = 0;
+	ret = sscanf(val, "%31[^:]:%lu", mode_nm, &meminfo_val);
+	if (ret != 2 && ret != 1)
+		return ERR_INVAL;
+	if ((mode = get_meminfo_mode(mode_nm)) < 0)
+		return ERR_INVAL;
+	if ((mode != VE_MEMINFO_NONE && ret !=2) ||
+	    (mode == VE_MEMINFO_NONE && ret == 2))
+		return ERR_INVAL;
+	if((mode != VE_MEMINFO_NONE) && meminfo_val == 0)
+		return ERR_INVAL;
+	param->mode = mode;
+	param->val = meminfo_val;
+
+	return 0;
+}
+
+static int store_meminfo(vps_param *old_p, vps_param *vps_p, vps_config *conf,
+	list_head_t *conf_h)
+{
+	char buf[64];
+	const char *mode_nm;
+	meminfo_param *meminfo = &vps_p->res.meminfo;
+
+	if (conf->id != PARAM_MEMINFO || meminfo->mode < 0)
+		return 0;
+	mode_nm = get_meminfo_mode_nm(meminfo->mode);
+	if (mode_nm == NULL)
+		return 0;
+	if (meminfo->mode == VE_MEMINFO_NONE) {
+		snprintf(buf, sizeof(buf), "%s=\"%s\"",
+		conf->name, mode_nm);
+	} else {
+		snprintf(buf, sizeof(buf), "%s=\"%s:%lu\"",
+		conf->name, mode_nm, meminfo->val);
+	}
+	add_str_param(conf_h, buf);
+	return 0;
+}
 
 int parse_ub(vps_param *vps_p, char *val, int id, int divisor)
 {
@@ -511,16 +567,13 @@ int parse_ub(vps_param *vps_p, char *val, int id, int divisor)
 
 	if ((conf = conf_get_by_id(config, id)) == NULL)
 		return ERR_INVAL;
-	if ((res.res_id = get_ub_resid(conf->name)) < 0) 
-		return ERR_UNK;
 	if (divisor)
 		ret = parse_twoul_sfx(val, res.limit, divisor);
 	else
 		ret = parse_twoul(val, res.limit);
 	if (ret && ret != ERR_LONG_TRUNC)
 		return ret;
-	if (get_ub_res(&vps_p->res.ub, res.res_id) != NULL)
-		return ERR_DUP;
+	res.res_id = id;
 	if (add_ub_param(&vps_p->res.ub, &res))
 		return ERR_NOMEM;
 	return ret;
@@ -529,19 +582,42 @@ int parse_ub(vps_param *vps_p, char *val, int id, int divisor)
 static int store_ub(vps_param *old_p, vps_param *vps_p,
 	list_head_t *conf_h)
 {
-	int i;
-	const char *name;
+	const vps_config *conf;
+	ub_param *ub = &vps_p->res.ub;
 	char buf[128];
-	const ub_param *ubp = &vps_p->res.ub;
 
-	for (i = 0; i < ubp->num_res; i++) {
-		const ub_res *ub = ubp->ub;
-		if ((name = get_ub_name(ub[i].res_id)) == NULL)
-			continue;
-		snprintf(buf, sizeof(buf), "%s=\"%lu:%lu\"", name,
-			ub[i].limit[0], ub[i].limit[1]);
-		add_str_param(conf_h, buf);
-	}
+#define ADD_UB_PARAM(res, id)						\
+if (ub->res != NULL) {							\
+	conf = conf_get_by_id(config, id);				\
+	snprintf(buf, sizeof(buf), "%s=\"%lu:%lu\"", conf->name,	\
+		ub->res[0], ub->res[1]);				\
+	if (add_str_param(conf_h, buf))					\
+		return ERR_NOMEM;					\
+}
+
+	ADD_UB_PARAM(kmemsize, PARAM_KMEMSIZE)
+	ADD_UB_PARAM(lockedpages, PARAM_LOCKEDPAGES)
+	ADD_UB_PARAM(privvmpages, PARAM_PRIVVMPAGES)
+	ADD_UB_PARAM(shmpages, PARAM_SHMPAGES)
+	ADD_UB_PARAM(numproc, PARAM_NUMPROC)
+	ADD_UB_PARAM(physpages,	PARAM_PHYSPAGES)
+	ADD_UB_PARAM(vmguarpages, PARAM_VMGUARPAGES)
+	ADD_UB_PARAM(oomguarpages, PARAM_OOMGUARPAGES)
+	ADD_UB_PARAM(numtcpsock, PARAM_NUMTCPSOCK)
+	ADD_UB_PARAM(numflock, PARAM_NUMFLOCK)
+	ADD_UB_PARAM(numpty, PARAM_NUMPTY)
+	ADD_UB_PARAM(numsiginfo, PARAM_NUMSIGINFO)
+	ADD_UB_PARAM(tcpsndbuf,	PARAM_TCPSNDBUF)
+	ADD_UB_PARAM(tcprcvbuf, PARAM_TCPRCVBUF)
+	ADD_UB_PARAM(othersockbuf, PARAM_OTHERSOCKBUF)
+	ADD_UB_PARAM(dgramrcvbuf, PARAM_DGRAMRCVBUF)
+	ADD_UB_PARAM(numothersock, PARAM_NUMOTHERSOCK)
+	ADD_UB_PARAM(numfile, PARAM_NUMFILE)
+	ADD_UB_PARAM(dcachesize, PARAM_DCACHESIZE)
+	ADD_UB_PARAM(numiptent, PARAM_NUMIPTENT)
+	ADD_UB_PARAM(avnumproc, PARAM_AVNUMPROC)
+#undef ADD_UB_PARAM
+
 	return 0;
 }
 
@@ -777,7 +853,7 @@ static int store_tmpl(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 }
 
 /***********************Quota***************************/
-static int parse_dq(unsigned long **param, const char *val)
+static int parse_dq(unsigned long **param, const char *val, int sfx)
 {
 	int ret;
 	unsigned long *tmp;
@@ -785,7 +861,10 @@ static int parse_dq(unsigned long **param, const char *val)
 	tmp = malloc(sizeof(unsigned long) * 2);
 	if (tmp == NULL)
 		return ERR_NOMEM;
-	ret = parse_twoul(val, tmp);
+	if (sfx)
+		ret = parse_twoul_sfx(val, tmp, 1);
+	else
+		ret = parse_twoul(val, tmp);
 	if (ret && ret != ERR_LONG_TRUNC) {
 		free(tmp);
 		return ret;
@@ -1080,6 +1159,27 @@ static int store_misc(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 
 /****************** CPU**********************************/
 
+static int parse_cpulimit(unsigned long **param, const char *str)
+{
+	char *tail;
+	int ncpu;
+	unsigned long val;
+
+	errno = 0;
+	val = (int)strtoul(str, (char **)&tail, 10);
+	if (*tail == '%' && *(tail + 1) == '\0') {
+		if (val > 100)
+			return ERR_INVAL;
+		ncpu = get_num_cpu();
+		val *= ncpu;
+	} else if (*tail != '\0' || errno == ERANGE)
+		return ERR_INVAL;
+
+	*param = malloc(sizeof(unsigned long));
+	**param = val;
+	return 0;
+}
+
 static int store_cpu(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	list_head_t *conf_h)
 {
@@ -1112,6 +1212,157 @@ static int store_cpu(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	return 0;
 }
 
+static int parse_hwaddr(const char *str, char *addr)
+{
+	int i;
+	char buf[3];
+	char *endptr;
+
+	for (i = 0; i < ETH_ALEN; i++) {
+		buf[0] = str[3*i];
+		buf[1] = str[3*i+1];
+		buf[2] = '\0';
+		addr[i] = strtol(buf, &endptr, 16);
+		if (*endptr != '\0')
+			return ERR_INVAL;
+	}
+	return 0;
+}
+
+static int parse_veth_str(const char *str, veth_dev *dev, int operation)
+{
+	char *ch, *tmp;
+	int len, err;
+
+	memset(dev, 0, sizeof(*dev));
+
+	if (!operation) {
+		/* Removing veth device */
+		/* Parsing veth device name in VE0 */
+		if ((ch = strchr(str, ',')) != NULL)
+			return ERR_INVAL;
+		len = strlen(str) + 1;
+		if (len > IFNAMSIZE)
+			return ERR_INVAL; 
+		snprintf(dev->dev_name, len, "%s", str);
+		return 0;
+	}
+	/* Creating veth device */
+	/* Parsing veth device name in VE0 */
+	if ((ch = strchr(str, ',')) == NULL)
+		return ERR_INVAL;
+	ch++;
+	len = ch - str;
+	if (len > IFNAMSIZE)
+		return ERR_INVAL;
+	snprintf(dev->dev_name, len, "%s", str);
+	tmp = ch;
+
+	/* Parsing veth MAC address in VE0 */
+	if ((ch = strchr(tmp, ',')) == NULL) 
+		return ERR_INVAL;
+	len = ch - tmp;
+	if (len != 3*ETH_ALEN -1)
+		return ERR_INVAL;
+	dev->addrlen = ETH_ALEN;
+	err = parse_hwaddr(tmp, dev->dev_addr);
+	if (err)
+		return ERR_INVAL;
+	ch++;
+	tmp = ch;
+
+	/* Parsing veth name in VPS */
+	if ((ch = strchr(tmp, ',')) == NULL) 
+		return ERR_INVAL;
+	ch++;
+	len = ch - tmp;
+	if (len > IFNAMSIZE)
+		return ERR_INVAL;
+	snprintf(dev->dev_name_ve, len, "%s", tmp);
+
+	/* Parsing veth MAC address in VPS */
+	len = strlen(ch);
+	if (len != 3*ETH_ALEN -1)
+		return ERR_INVAL;
+	dev->addrlen_ve = ETH_ALEN;
+	err = parse_hwaddr(ch, dev->dev_addr_ve);
+	if (err)
+		return ERR_INVAL;
+
+	return 0;
+}
+
+static int parse_veth(vps_param *vps_p, char *val, int operation)
+{
+	int ret;
+	char *token;
+	veth_dev dev;
+
+	if ((token = strtok(val, " ")) == NULL)
+		return 0;
+	do {
+		if (parse_veth_str(token, &dev, operation))
+			return ERR_INVAL;
+		if (operation)
+			ret = add_veth_param(&vps_p->res.veth, &dev);
+		else
+			ret = add_veth_param(&vps_p->del_res.veth, &dev);
+
+	} while ((token = strtok(NULL, " ")));
+
+	return 0;
+}
+
+static int store_veth(vps_param *old_p, vps_param *vps_p, vps_config *conf,
+        list_head_t *conf_h)
+{
+	char buf[STR_SIZE];
+	veth_dev *dev;
+	int r;
+	char *sp, *ep;
+	veth_param merged;
+
+#define STR2MAC(dev)			\
+	((unsigned char *)dev)[0],	\
+	((unsigned char *)dev)[1],	\
+	((unsigned char *)dev)[2],	\
+	((unsigned char *)dev)[3],	\
+	((unsigned char *)dev)[4],	\
+	((unsigned char *)dev)[5]
+
+	if (conf->id != PARAM_VETH_ADD)
+		return 0;
+	if (list_empty(&vps_p->res.veth.dev) &&
+		list_empty(&vps_p->del_res.veth.dev))
+	{
+		return 0;
+	}
+	list_head_init(&merged.dev);
+	if (merge_veth_list(&old_p->res.veth.dev, &vps_p->res.veth.dev,
+		&vps_p->del_res.veth.dev, &merged))
+	{
+		return ERR_NOMEM;
+	}
+	sp = buf;
+	ep = buf + sizeof(buf) - 1;
+	sp += snprintf(buf, sizeof(buf), "%s=\"", conf->name);
+	list_for_each(dev, &merged.dev, list) {
+		r = snprintf(sp, ep - sp,
+			"%s,%02X:%02X:%02X:%02X:%02X:%02X,"
+			"%s,%02X:%02X:%02X:%02X:%02X:%02X ",
+			dev->dev_name, STR2MAC(dev->dev_addr),
+			dev->dev_name_ve, STR2MAC(dev->dev_addr_ve));
+		sp += r;
+		if ((r < 0) || (sp >= ep))
+			break;
+	}
+	strcat(buf, "\"");
+	add_str_param(conf_h, buf);
+	free_veth_param(&merged);
+	return 0;
+#undef STR2MAC
+}
+
 static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 {
 	int ret;
@@ -1134,6 +1385,9 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_SAVE:
 		vps_p->opt.save = YES;
 		break;
+	case PARAM_RESET_UB:
+		vps_p->opt.reset_ub = YES;
+		break;
 	case PARAM_ONBOOT:
 		ret = conf_parse_yesno(&vps_p->opt.onboot, val, 1);
 		break;
@@ -1202,7 +1456,7 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_TCPRCVBUF:
 	case PARAM_OTHERSOCKBUF:
 	case PARAM_DGRAMRCVBUF:
-	case PARAM_DCACHE:
+	case PARAM_DCACHESIZE:
 		ret = parse_ub(vps_p, val, id, 1);
 		break;
 	case PARAM_CAP:
@@ -1253,13 +1507,13 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_DISKSPACE:
 		if (vps_p->res.dq.diskspace != NULL)
 			break;
-		if (parse_dq(&vps_p->res.dq.diskspace, val))
+		if (parse_dq(&vps_p->res.dq.diskspace, val, 1))
 			ret = ERR_INVAL;
 		break;
 	case PARAM_DISKINODES:
 		if (vps_p->res.dq.diskinodes != NULL)
 			break;
-		if (parse_dq(&vps_p->res.dq.diskinodes, val))
+		if (parse_dq(&vps_p->res.dq.diskinodes, val, 0))
 			ret = ERR_INVAL;
 		break;
 	case PARAM_QUOTATIME:
@@ -1313,12 +1567,17 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_CPULIMIT:
 		if (vps_p->res.cpu.limit != NULL)
 			break;
-	        if (parse_ul(val, &uid))
+	        if (parse_cpulimit(&vps_p->res.cpu.limit, val))
         	        return ERR_INVAL;
-		vps_p->res.cpu.limit = malloc(sizeof(unsigned long));
-		if (vps_p->res.cpu.limit == NULL)
-			return ERR_NOMEM;
-		*vps_p->res.cpu.limit = uid;
+		break;
+	case PARAM_MEMINFO:
+		ret = parse_meminfo(&vps_p->res.meminfo, val);
+		break;
+	case PARAM_VETH_ADD:
+		ret = parse_veth(vps_p, val, 1);
+		break;
+	case PARAM_VETH_DEL:
+		ret = parse_veth(vps_p, val, 0);
 		break;
 	default:
 		logger(10, 0, "Not handled parameter %d %s", id, val);
@@ -1344,6 +1603,8 @@ static int store(vps_param *old_p, vps_param *vps_p, list_head_t *conf_h)
 		store_devnodes(old_p, vps_p, conf, conf_h);
 		store_misc(old_p, vps_p, conf, conf_h);
 		store_cpu(old_p, vps_p, conf, conf_h);
+		store_meminfo(old_p, vps_p, conf, conf_h);
+		store_veth(old_p, vps_p, conf, conf_h);
 	}
 	return 0;
 }
@@ -1591,6 +1852,7 @@ vps_param *init_vps_param()
 	list_head_init(&param->res.misc.nameserver);
 	list_head_init(&param->res.misc.searchdomain);
 	list_head_init(&param->res.dev.dev);
+	list_head_init(&param->res.veth.dev);
 
 	list_head_init(&param->del_res.net.ip);
 	list_head_init(&param->del_res.net.dev);
@@ -1599,6 +1861,8 @@ vps_param *init_vps_param()
 	list_head_init(&param->del_res.misc.nameserver);
 	list_head_init(&param->del_res.misc.searchdomain);
 	list_head_init(&param->del_res.dev.dev);
+	list_head_init(&param->del_res.veth.dev);
+	param->res.meminfo.mode = -1;
 
 	return param;
 }
@@ -1693,6 +1957,7 @@ static void free_vps_res(vps_res *res)
 	free_cpu(&res->cpu);
 	free_dq(&res->dq);
 	free_cpt(&res->cpt);
+	free_veth_param(&res->veth);
 }
 
 void free_vps_param(vps_param *param)
@@ -1838,6 +2103,20 @@ static void merge_cpt(cpt_param *dst, cpt_param *src)
 	MERGE_INT(cmd);
 }
 
+static void merge_meminfo(meminfo_param *dst, meminfo_param *src)
+{
+	MERGE_INT(mode);
+	MERGE_INT(val);
+}
+
+static void merge_veth(veth_param *dst, veth_param *src)
+{
+	if (!list_empty(&src->dev)) {
+		free_veth_param(dst);
+		copy_veth_param(dst, src);
+	}
+}
+
 static int merge_res(vps_res *dst, vps_res *src)
 {
 	merge_fs(&dst->fs, &src->fs);
@@ -1851,6 +2130,8 @@ static int merge_res(vps_res *dst, vps_res *src)
 	merge_dev(&dst->dev, &src->dev);
 	merge_env(&dst->env, &src->env);
 	merge_cpt(&dst->cpt, &src->cpt);
+	merge_meminfo(&dst->meminfo, &src->meminfo);
+	merge_veth(&dst->veth, &src->veth);
 	return 0;
 }
 
