@@ -124,6 +124,27 @@ int vz_env_create_ioctl(vps_handler *h, envid_t veid, int flags)
 	return errcode;
 }
 
+/*
+ * Reset standart file descriptord to /dev/null in case they are closed.
+ */
+static int reset_std()
+{
+	int ret, i, stdfd;
+
+	stdfd = -1;
+	for (i = 0; i < 3; i++) {
+		ret = fcntl(i, F_GETFL);
+		if (ret < 0 && errno == EBADF) {
+			if (stdfd < 0) {
+				if ((stdfd = open("/dev/null", O_RDWR)) < 0)
+					return -1;
+			}
+			dup2(stdfd, i);
+		}
+	}
+	return stdfd;
+}
+
 /** Allocate and inittialize VPS handler.
  *
  * @param veid		VPS id.
@@ -131,20 +152,22 @@ int vz_env_create_ioctl(vps_handler *h, envid_t veid, int flags)
  */
 vps_handler *vz_open(envid_t veid)
 {
-	int vzfd;
+	int vzfd, stdfd;
 	vps_handler *h = NULL;
 
+	stdfd = reset_std();
         if ((vzfd = open(VZCTLDEV, O_RDWR)) < 0) {
                 logger(0, errno, "Unable to open %s", VZCTLDEV);
                 logger(0, 0, "Please check that vzdev kernel module is loaded"
                         " and you have sufficient permissions"
                         " to access the file.");
-		return NULL;
+		goto err;
         }
 	h = calloc(1, sizeof(*h));
 	if (h == NULL) 
 		goto err;
 	h->vzfd = vzfd;
+	h->stdfd = stdfd;
         if (vz_env_create_ioctl(h, 0, 0) < 0 &&
 		(errno == ENOSYS || errno == EPERM))
         {
@@ -157,7 +180,10 @@ vps_handler *vz_open(envid_t veid)
 err:
 	if (h != NULL)
 		free(h);
-	close(vzfd);
+	if (vzfd != -1)
+		close(vzfd);
+	if (stdfd != -1)
+		close(stdfd);
 	return NULL;
 }
 
@@ -170,6 +196,8 @@ void vz_close(vps_handler *h)
 	if (h == NULL)
 		return;
 	close(h->vzfd);
+	if (h->stdfd != -1)
+		close(h->stdfd);
 	free(h);
 }
 
@@ -185,6 +213,8 @@ int vps_is_run(vps_handler *h, envid_t veid)
 	int ret;
 	int errno;
 
+	if (veid == 0)
+		return 1;
 	ret = vz_env_create_ioctl(h, veid, VE_TEST);
 
 	if (ret < 0 && (errno == ESRCH || errno == ENOTTY))
@@ -283,7 +313,7 @@ static int _env_create(vps_handler *h, envid_t veid, int wait_p, int err_p,
 #endif
 	int fd, ret;
 	vps_res *res;
-	char *argv[] = {"init", NULL};
+	char *argv[] = {"init", "-z", "      ", NULL};
 	char *envp[] = {"HOME=/", "TERM=linux", NULL};
 	int retry = 0;
 
@@ -371,7 +401,7 @@ static int vz_real_env_create(vps_handler *h, envid_t veid, vps_res *res,
 		return ret;
 	if ((ret = vz_setluid(veid)))
 		return ret;
-	if ((ret = set_ublimit(h, veid, &res->ub)))
+	if ((ret = setup_resource_management(h, veid, res)))
 		return ret;
 	/* Create another process to proper resource accounting */
 	if ((pid = fork()) < 0) {
@@ -467,19 +497,17 @@ err:
 static void fix_numiptent(ub_param *ub)
 {
 	unsigned long min_ipt;
-        ub_res *res;
 
-	res = get_ub_res(ub, UB_IPTENTRIES);
-	if (res == NULL)
+	if (ub->numiptent == NULL)
 		return;
-	min_ipt = min_ul(res->limit[0], res->limit[1]);
-        if (min_ipt < MIN_NUMIPTENT) {
+	min_ipt = min_ul(ub->numiptent[0], ub->numiptent[1]);
+	if (min_ipt < MIN_NUMIPTENT) {
 		logger(0, 0, "Warning: NUMIPTENT %d:%d is less"
 			" than minimally allowable value, set to %d:%d",
-                        res->limit[0], res->limit[1],
-                        MIN_NUMIPTENT, MIN_NUMIPTENT);
-		res->limit[0] = MIN_NUMIPTENT;
-		res->limit[1] = MIN_NUMIPTENT;
+			ub->numiptent[0], ub->numiptent[1],
+			MIN_NUMIPTENT, MIN_NUMIPTENT);
+		ub->numiptent[0] = MIN_NUMIPTENT;
+		ub->numiptent[1] = MIN_NUMIPTENT;
 	}
 }
 
@@ -505,18 +533,20 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 	dist_actions actions;
 
 	memset(&actions, 0, sizeof(actions));
-	dist_name = get_dist_name(&res->tmpl);
-	ret = read_dist_actions(dist_name, DIST_DIR, &actions);
-	if (dist_name != NULL)
-		free(dist_name);
-	if (ret)
-		return ret;
 	if (check_var(res->fs.root, "VE_ROOT is not set"))
 		return VZ_VE_ROOT_NOTSET;
 	if (vps_is_run(h, veid)) {
 		logger(0, 0, "VPS is already running");
 		return VZ_VE_RUNNING;
 	}
+	if ((ret = check_ub(&res->ub)))
+		return ret;
+	dist_name = get_dist_name(&res->tmpl);
+	ret = read_dist_actions(dist_name, DIST_DIR, &actions);
+	if (dist_name != NULL)
+		free(dist_name);
+	if (ret)
+		return ret;
 	logger(0, 0, "Starting VPS ...");
 	if (vps_is_mounted(res->fs.root)) {
 		/* if VPS mounted umount first, to cleanup mount state */
@@ -729,11 +759,9 @@ int vps_stop(vps_handler *h, envid_t veid, vps_param *param, int stop_mode,
 	skipFlags skip, struct mod_action *action)
 {
 	int ret;
-	list_head_t ips;
 	char buf[64];
 	vps_res *res = &param->res;
 
-	list_head_init(&ips);
 	if (check_var(res->fs.root, "VE_ROOT is not set"))
 		return VZ_VE_ROOT_NOTSET;
 	if (!vps_is_run(h, veid)) {
@@ -751,17 +779,30 @@ int vps_stop(vps_handler *h, envid_t veid, vps_param *param, int stop_mode,
 			}
 		}
 	}
-	get_vps_ip(h, veid, &ips);
+	/* get VPS ip adresses for cleanup */
+	get_vps_ip(h, veid, &param->del_res.net.ip);
 	if ((ret = env_stop(h, veid, res->fs.root, stop_mode)))
 		return ret;
 	mod_cleanup(h, veid, action, param);
-	if (!res->net.skip_route_cleanup)
-		run_net_script(veid, DEL, &ips, STATE_RUNNING,
-			param->res.net.skip_arpdetect);
+	vps_cleanup_res(h, veid, param, STATE_STOPPING);
 	ret = vps_umount(h, veid, res->fs.root, skip);
 	/* Clear VPS network configuration*/
 	ret = run_pre_script(veid, VPS_STOP);
-	free_str_param(&ips);
+
+	return ret;
+}
+
+int vps_restart(vps_handler *h, envid_t veid, vps_param *param)
+{
+	int ret;
+
+	logger(0, 0, "Restarting VPS");
+	if (vps_is_run(h, veid) &&
+		(ret = vps_stop(h, veid, param, M_REBOOT, SKIP_NONE, NULL)))
+	{
+		return ret;
+	}
+	ret = vps_start(h, veid, param, SKIP_NONE, NULL);
 
 	return ret;
 }
