@@ -20,6 +20,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <getopt.h>
+#include <signal.h>
 
 #include "vzctl.h"
 #include "env.h"
@@ -74,8 +75,10 @@ void usage()
 "   [--noatime yes|no] [--capability <name>:on|off ...]\n"
 "   [--devices b|c:major:minor|all:r|w|rw]\n"
 "   [--devnodes device:r|w|rw|none]\n"
+"   [--veth_add dev_name,dev_addr,dev_name_vps,dev_addr_vps]\n"
+"   [--veth_del dev_name]\n"
 "   [--applyconfig <name>]\n");
-   
+ 
 	fprintf(stdout, "   [--iptables <name>] [--disabled <yes|no>]\n");
 	fprintf(stdout, "   [UBC parameters]\n"
 "UBC parameters (N - items, P - pages, B - bytes):\n"
@@ -104,6 +107,7 @@ int main(int argc, char *argv[], char *envp[])
 	char buf[256];
 	vps_param *gparam = NULL, *vps_p = NULL, *cmd_p = NULL;
 	const char *action_nm;
+	struct sigaction act;
 
 	_proc_title = argv[0];
 	_proc_title_len = envp[0] - argv[0];
@@ -112,6 +116,11 @@ int main(int argc, char *argv[], char *envp[])
 	vps_p = init_vps_param();
 	cmd_p = init_vps_param();
 
+	sigemptyset(&act.sa_mask);
+	act.sa_handler = SIG_IGN;
+	act.sa_flags = 0;
+	sigaction(SIGPIPE, &act, NULL);
+
 	while (argc > 1) {
 		if (!strcmp(argv[1], "--verbose")) {
 			verbose = 2;
@@ -221,14 +230,11 @@ int main(int argc, char *argv[], char *envp[])
 	{
 		goto error;
 	}
-	if (veid == 0 && action == ACTION_SET) {
-		if (!cmd_p->res.cpu.units) {
-			fprintf(stderr, "Only option cpuunits can be used in"
-				" VE0\n");
-                        ret = VZ_INVALID_PARAMETER_VALUE;
-			goto error;
-		}
-	} else if (veid <= 0) {
+	if (veid == 0 && action != ACTION_SET) {
+		fprintf(stderr, "Only set actions allowed for VE0");
+		ret = VZ_INVALID_PARAMETER_VALUE;
+		goto error;
+	} else if (veid < 0) {
 		fprintf(stderr, "Bad VPS id %d\n", veid);
 		ret = VZ_INVALID_PARAMETER_VALUE;
 		goto error;
