@@ -40,6 +40,7 @@
 #include "res.h"
 #include "logger.h"
 #include "util.h"
+#include "types.h"
 
 LOG_DATA
 static struct Cveinfo *veinfo = NULL; 
@@ -758,18 +759,6 @@ char *parse_var(char *var)
 	return strdup(sp);
 }
 
-#define MERGE_UBC(name, ubc, res)				\
-do {								\
-	ub_res *ub;						\
-	int id;							\
-	if ((id = get_ub_resid("" #name "")) < 0)		\
-		break;						\
-	if ((ub = get_ub_res(&res->ub, id)) == NULL)		\
-		break;						\
-	ubc->name[2] = ub->limit[0];     			\
-	ubc->name[3] = ub->limit[1];				\
-} while(0);							
-
 #define MERGE_QUOTA(name, quota, dq)				\
 do {								\
 	if (dq.name != NULL) {					\
@@ -783,6 +772,13 @@ void merge_conf(struct Cveinfo *ve, vps_res *res)
 	if (ve->ubc == NULL) {
 		ve->ubc = x_malloc(sizeof(struct Cubc));
 		memset(ve->ubc, 0, sizeof(struct Cubc));
+#define MERGE_UBC(name, ubc, res)				\
+do {								\
+	if (res == NULL || res->ub.name == NULL) 		\
+		break;						\
+	ubc->name[2] = res->ub.name[0];     			\
+	ubc->name[3] = res->ub.name[1];				\
+} while(0);							
 
 		MERGE_UBC(kmemsize, ve->ubc, res);
 		MERGE_UBC(lockedpages, ve->ubc, res);
@@ -804,6 +800,7 @@ void merge_conf(struct Cveinfo *ve, vps_res *res)
 		MERGE_UBC(dcachesize, ve->ubc, res);
 		MERGE_UBC(numfile, ve->ubc, res);
 		MERGE_UBC(numiptent, ve->ubc, res);
+#undef MERGE_UBC
 	}
 	if (ve->ip == NULL && !list_empty(&res->net.ip)) {
 		ve->ip = strdup(list2str(NULL, &res->net.ip));
@@ -848,26 +845,30 @@ int read_ves_param()
 
 	param = init_vps_param();
 	/* Parse global config file */
-	vps_parse_config(0, CFG_FILE, param, NULL);
+	vps_parse_config(0, GLOBAL_CFG, param, NULL);
 	if (param->res.fs.root != NULL)
-		ve_root = strdup(param->res.fs.root);
+		ve_root = strdup(param->res.fs.root_orig);
 	if (param->res.fs.private != NULL)
-		ve_private = strdup(param->res.fs.private);
+		ve_private = strdup(param->res.fs.private_orig);
 	free_vps_param(param);
 	for (i = 0; i < n_veinfo; i++) { 
 		veid = veinfo[i].veid;
 		param = init_vps_param();
-		snprintf(buf, sizeof(buf), SCRIPT_DIR "%d.conf", veid);
+		snprintf(buf, sizeof(buf), VPS_CONF_DIR "%d.conf", veid);
 		vps_parse_config(veid, buf, param, NULL);
 		merge_conf(&veinfo[i], &param->res);
 		if (veinfo[i].ve_root == NULL)
-			veinfo[i].ve_root = strdup(ve_root);
+			veinfo[i].ve_root = subst_VEID(veinfo[i].veid, ve_root);
 		if (veinfo[i].ve_private == NULL)
-			veinfo[i].ve_private = strdup(ve_private);
+			veinfo[i].ve_private = subst_VEID(veinfo[i].veid,
+								ve_private);
 		free_vps_param(param);
 	}
 	if (ve_root != NULL)
 		free(ve_root);
+	if (ve_private != NULL)
+		free(ve_private);
+
 	return 0;
 }
 
@@ -1344,7 +1345,7 @@ int get_ve_list()
 	struct Cveinfo ve;
 	char str[6];
 
-	dp = opendir(SCRIPT_DIR);
+	dp = opendir(VPS_CONF_DIR);
 	if (dp == NULL) {
 		return -1;
 	}
